<?php
namespace Mig\ImportacaoXLS;

use Doctrine\ORM\EntityManager;
use Mig\ImportacaoXLS\Logic\ImportacaoXLS;

use Util\Environment;
use Util\Messages;

use Mig\ImportacaoXLS\Importador;

use Util\{
    DateUtil,
    //FileUploader,
    //HCMParameters,
    Repositories,
    Util
};

use Teknisa\Model\Gestaopessoas\Entities\{GpeAnexos, GpeMigracao};
use \Zeedhi\Framework\ORM\DateTime;

class Importador
{
    /** @var Environment */
    protected $environment;
    
    public function __construct($row, $environment, $entityManager)
    {
        $this->row = $row;
        $this->environment = $environment;
        $this->entityManager = $entityManager;

        $this->parametrosImportador = new ParametrosImportador(
            $this->row,
            $this->environment
        );

        $this->importacaoXls = new Importacao(
            $this->parametrosImportador->jsonPath(),
            $this->parametrosImportador->environment(),
            $this->entityManager,
            $this->parametrosImportador->nomeImportacao(),
            $this->parametrosImportador->errorsIgnore(),
            $this->parametrosImportador->importarDados()
        );

        $this->statusMigracao = new StatusMigracao(
            $this->entityManager,
            $this->parametrosImportador,
            $this->importacaoXls
        );

        $this->pendenciasAnteriores = new PendenciasAnteriores(
            $this->entityManager,
            $this->parametrosImportador
        );
    }

    public function importarPlanilha(): void
    {
        $this->verifyPendenciasAnteriores();

        $this->verifyPlanilhaExists();

        File::downloadMidia(
            $this->parametrosImportador->nomePlanilha(),
            $this->parametrosImportador->urlPlanilha()
        );
        
        $this->importacaoXls->importar(
            $this->parametrosImportador->nomePlanilha()
        );

        $this->statusMigracao->atualizar();
    }

    private function verifyPendenciasAnteriores(): void
    {
        if ($this->pendenciasAnteriores->exists()) {
            throw new \Exception(
                "Existem pendências nas importações anteriores"
            );
        }
    }

    private function verifyPlanilhaExists(): void
    {
        $planilhaDontExists =
            is_null($this->row["DSARQUIVO"]) ||
            $this->row["DSARQUIVO"] == "" ||
            $this->row["DSARQUIVO"] == "null";

        if ($planilhaDontExists) {
            throw new \Exception(
                "Nenhuma planilha foi encontrada para importação!"
            );
        }
    }

    public function errosImportacaoPlanilha(): array
    {
        $erros = $this->importacaoXls->getErrosImportacao();

        return $erros ?? [];
    }
}

class StatusMigracao
{
    public function __construct(
        $entityManager,
        $parametrosImportador,
        $importacaoXls
    ) {
        $this->entityManager = $entityManager;
        $this->parametrosImportador = $parametrosImportador;

        $this->findByParams = [
            "nrorg" => $this->parametrosImportador->environment()["NRORG"],
            "nrmigracao" => $this->parametrosImportador->nrMigracao(),
        ];

        $this->importacaoXls = $importacaoXls;

        $this->gpeMigracao = $this->entityManager
            ->getRepository(Repositories::GPE_MIGRACAO)
            ->findOneBy($this->findByParams);
    }

    public function atualizar()
    {
        $this->entityManager->beginTransaction();

        $this->setStatusMigracao();
        $this->setQtdLinhasPendentes();
        //$this->setQtdLinhasImportadas();

        $this->entityManager->flush();
        $this->entityManager->commit();
    }

    private function setStatusMigracao(): void
    {
        if (!empty($this->errosImportacaoPlanilha())) {
            $this->gpeMigracao->setIdstatus("E");
        } else {
            $this->gpeMigracao->setIdstatus("C");
        }
    }

    private function setQtdLinhasPendentes()
    {
        $erros = $this->errosImportacaoPlanilha();

        if (!empty($erros)) {
            $this->gpeMigracao->setQtdadospendentes(count($erros));
        } else {
            $this->gpeMigracao->setQtdadospendentes(0);
        }
    }

    private function setQtdLinhasImportadas()
    {
        $this->gpeMigracao->setQtdadosimportados(
            $this->importacaoXls->getQtdLinhasImportadas()
        );
    }

    private function errosImportacaoPlanilha(): array
    {
        $erros = $this->importacaoXls->getErrosImportacao();

        return $erros ?? [];
    }
}

class ParametrosImportador
{
    public function __construct($row, $environment)
    {
        $this->row = $row;
        $this->environment = $environment;
        
        $this->row["DSMIGRACAO"] = Normalize::noAccents($row["DSMIGRACAO"]);
        $this->row["DSMIGRACAO"] = Normalize::noSlash(
            $this->row["DSMIGRACAO"]
        );
        $this->row["DSMIGRACAO"] = Normalize::camelCase($row["DSMIGRACAO"]);
    }

    public function jsonPath(): string
    {
        return Path::IMPORTACAO . $this->row["DSMIGRACAO"] . ".json";
    }

    public function nomePlanilha(): string
    {
        return explode("/", $this->row["DSARQUIVO"])[5];
    }

    public function urlPlanilha(): string
    {
        return $this->row["DSARQUIVO"];
    }

    public function nomeImportacao(): string
    {
        return $this->row["DSMIGRACAO"];
    }

    public function errorsIgnore(): bool
    {
        return $this->row["errorsIgnore"];
    }
    
    public function importarDados(): bool
    {
        return $this->row["importarDados"];
    }

    public function nrMigracao(): int
    {
        return $this->row["NRMIGRACAO"];
    }

    public function environment(): array
    {
        return [
            "NRORG" => $this->environment->getNrOrgTrab(),
            "NRORG_PADRAO" => $this->environment->getNrorgPadrao(),
            "CDOPERADOR" => $this->environment->getCdoperador(),
            "COMPETENCIA" => $this->row['DTCOMPET'],
            "NRMIGRACAO" => $this->row["NRMIGRACAO"],
        ];
    }
}

class File
{
    public static function downloadMidia($fileName, $fileUrl): void
    {
        $fh = fopen(Path::TEMP . $fileName, "w");
        $ch = curl_init();

        curl_setopt(
            $ch,
            CURLOPT_URL,
            Util::getFileServerUrl() . $fileUrl
        );
        curl_setopt($ch, CURLOPT_FILE, $fh);
        curl_exec($ch);
        curl_close($ch);
    }
}

class Path
{
    const TEMP = "/tmp/";
    const IMPORTACAO = "/ImportacaoXLS/";
}

class Log
{
    private $path;

    public function __construct($path)
    {
        $this->path = $path;
    }

    public function get()
    {
        $stream = fopen($this->path, "r");

        $data = stream_get_contents($stream);

        fclose($stream);

        return $data;
    }
}

class Normalize
{
    private const UTF8 = [
        "/[áàâãªä]/u" => "a",
        "/[ÁÀÂÃÄ]/u" => "A",
        "/[ÍÌÎÏ]/u" => "I",
        "/[íìîï]/u" => "i",
        "/[éèêë]/u" => "e",
        "/[ÉÈÊË]/u" => "E",
        "/[óòôõºö]/u" => "o",
        "/[ÓÒÔÕÖ]/u" => "O",
        "/[úùûü]/u" => "u",
        "/[ÚÙÛÜ]/u" => "U",
        "/ç/" => "c",
        "/Ç/" => "C",
        "/ñ/" => "n",
        "/Ñ/" => "N",
        "/–/" => "-", // UTF-8 hyphen to "normal" hyphen
        "/[’‘‹›]/u" => " ", // Literally a single quote
        "/[“”«»„]/u" => " ", // Double quote
        "/ /" => " ", // nonbreaking space (equiv. to 0x160)
    ];

    public static function noAccents($string)
    {
        return preg_replace(
            array_keys(self::UTF8),
            array_values(self::UTF8),
            $string
        );
    }

    public static function camelCase($string)
    {
        return str_replace(" ", "", ucwords(str_replace("_", " ", $string)));
    }

    public static function noSlash($string)
    {
        return preg_replace(
            "/[\/]/",
            "",
            $string
        );
    }
}

class Importacao extends ImportacaoXLS
{
    public function importar($nomePlanilha): void
    {
        $this->loadPlanilha(Path::TEMP . $nomePlanilha);

        unlink(Path::TEMP . $nomePlanilha);
        
        $this->checkAllDataTyping();
        $this->applyFieldFormat();
        //$this->checkFkValues();
        
        if($this->importarDados){
            
            //$this->insertDados();    
        }
        
    }
}

class PendenciasAnteriores
{
    public function __construct($entityManager, $parametrosImportador)
    {
        $this->entityManager = $entityManager;
        $this->parametrosImportador = $parametrosImportador;
    }

    public function exists(): bool
    {
        return count($this->get()) > 0;
    }

    private function get(): array
    {
        $query = 'SELECT 1
                    FROM GPE_MIGRACAO
                   WHERE NRORG = :P_NRORG AND NRMIGRACAO NOT IN (:P_NRMIGRACAO) AND NRMIGRACAO < :P_NRMIGRACAO AND IDSTATUS = :P_IDSTATUS';

        $sql = $this->entityManager->getConnection()->prepare($query);

        $binds = [
            "P_NRORG" => $this->parametrosImportador->environment()["NRORG"],
            "P_NRMIGRACAO" => $this->parametrosImportador->nrMigracao(),
            "P_IDSTATUS" => "E"
        ];

        $sql->execute($binds);

        return $sql->fetchAll();
    }
}
